@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 Derick Rethans                               |
+   | Copyright (c) 1997-2019 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -65,6 +65,9 @@ void vld_branch_info_update(vld_branch_info *branch_info, unsigned int pos, unsi
 void vld_only_leave_first_catch(zend_op_array *opa, vld_branch_info *branch_info, int position)
 {
 	unsigned int exit_jmp;
+#if PHP_VERSION_ID >= 70300 && ZEND_USE_ABS_JMP_ADDR
+	zend_op *base_address = &(opa->opcodes[0]);
+#endif
 
 	if (opa->opcodes[position].opcode == ZEND_FETCH_CLASS) {
 		position++;
@@ -74,13 +77,17 @@ void vld_only_leave_first_catch(zend_op_array *opa, vld_branch_info *branch_info
 		return;
 	}
 
-	if (!opa->opcodes[position].result.num) {
-#if PHP_VERSION_ID >= 70100
-		exit_jmp = position + ((signed int) opa->opcodes[position].extended_value / sizeof(zend_op));
+#if PHP_VERSION_ID >= 70300
+	if (!(opa->opcodes[position].extended_value & ZEND_LAST_CATCH)) {
+		exit_jmp = VLD_ZNODE_JMP_LINE(opa->opcodes[position].op2, position, base_address);
 #else
+	if (!opa->opcodes[position].result.num) {
+# if PHP_VERSION_ID >= 70100
+		exit_jmp = position + ((signed int) opa->opcodes[position].extended_value / sizeof(zend_op));
+# else
 		exit_jmp = opa->opcodes[position].extended_value;
+# endif
 #endif
-
 		if (opa->opcodes[exit_jmp].opcode == ZEND_FETCH_CLASS) {
 			exit_jmp++;
 		}
@@ -96,12 +103,17 @@ void vld_branch_post_process(zend_op_array *opa, vld_branch_info *branch_info)
 {
 	unsigned int i;
 	int in_branch = 0, last_start = VLD_JMP_NOT_SET;
+#if PHP_VERSION_ID >= 70300 && ZEND_USE_ABS_JMP_ADDR
+	zend_op *base_address = &(opa->opcodes[0]);
+#endif
 
 	/* Figure out which CATCHes are chained, and hence which ones should be
 	 * considered entry points */
 	for (i = 0; i < branch_info->entry_points->size; i++) {
 		if (vld_set_in(branch_info->entry_points, i) && opa->opcodes[i].opcode == ZEND_CATCH) {
-#if PHP_VERSION_ID >= 70100
+#if PHP_VERSION_ID >= 70300
+			vld_only_leave_first_catch(opa, branch_info, VLD_ZNODE_JMP_LINE(opa->opcodes[i].op2, i, base_address));
+#elif PHP_VERSION_ID >= 70100
 			vld_only_leave_first_catch(opa, branch_info, i + ((signed int) opa->opcodes[i].extended_value / sizeof(zend_op)));
 #else
 			vld_only_leave_first_catch(opa, branch_info, opa->opcodes[i].extended_value);
