@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 Derick Rethans                               |
+   | Copyright (c) 1997-2019 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -15,7 +15,6 @@
 
 #include "php.h"
 #include "zend_alloc.h"
-#include "zend_compile.h"
 #include "branchinfo.h"
 #include "srm_oparray.h"
 #include "ext/standard/url.h"
@@ -24,12 +23,6 @@
 
 ZEND_EXTERN_MODULE_GLOBALS(vld)
 
-#if ZEND_USE_ABS_JMP_ADDR
-# define VLD_ZNODE_JMP_LINE(node, opline, base)  (int32_t)(((long)((node).jmp_addr) - (long)(base_address)) / sizeof(zend_op))
-#else
-# define VLD_ZNODE_JMP_LINE(node, opline, base)  (int32_t)(((int32_t)((node).jmp_offset) / sizeof(zend_op)) + (opline))
-#endif
-
 /* Input zend_compile.h
  * And replace [^(...)(#define )([^ \t]+).*$]
  * BY     [/=*  \1 *=/  { "\3", ALL_USED },] REMEMBER to remove the two '=' signs
@@ -143,9 +136,19 @@ static const op_usage opcodes[] = {
 	/*  95 */	{ "FETCH_UNSET", ALL_USED },
 	/*  96 */	{ "FETCH_DIM_UNSET", ALL_USED },
 	/*  97 */	{ "FETCH_OBJ_UNSET", ALL_USED },
+#if PHP_VERSION_ID >= 70300
+	/*  98 */	{ "FETCH_LIST_R", ALL_USED },
+#else
 	/*  98 */	{ "FETCH_LIST", ALL_USED },
+#endif
 	/*  99 */	{ "FETCH_CONSTANT", ALL_USED },
-	/*  100 */	{ "GOTO", ALL_USED | OP1_OPLINE },
+
+#if PHP_VERSION_ID >= 70300
+	/*  100 */	{ "CHECK_FUNC_ARG", ALL_USED },
+#else
+	/*  100 */	{ "UNKNOWN [100]", ALL_USED },
+#endif
+
 	/*  101 */	{ "EXT_STMT", ALL_USED },
 	/*  102 */	{ "EXT_FCALL_BEGIN", ALL_USED },
 	/*  103 */	{ "EXT_FCALL_END", ALL_USED },
@@ -164,14 +167,10 @@ static const op_usage opcodes[] = {
 
 	/*  116 */	{ "SEND_VAL_EX", ALL_USED },
 	/*  117 */	{ "SEND_VAR", ALL_USED },
-	/*  118 */	{ "INIT_USER_CALL", ALL_USED | EXT_VAL },
 
-	/*  119 */	{ "UNKNOWN [119]", ALL_USED },
-#if PHP_VERSION_ID >= 70000
+	/*  118 */	{ "INIT_USER_CALL", ALL_USED | EXT_VAL },
+	/*  119 */	{ "SEND_ARRAY", ALL_USED },
 	/*  120 */	{ "SEND_USER", ALL_USED },
-#else
-	/*  120 */	{ "UNKNOWN [120]", ALL_USED },
-#endif
 
 	/*  121 */	{ "STRLEN", ALL_USED },
 	/*  122 */	{ "DEFINED", ALL_USED },
@@ -239,7 +238,11 @@ static const op_usage opcodes[] = {
     /*  182 */ { "BIND_LEXICAL", ALL_USED },
     /*  183 */ { "BIND_STATIC", ALL_USED },
 	/*  184 */	{ "FETCH_THIS", ALL_USED },
+#if PHP_VERSION_ID >= 70300
+	/*  185 */	{ "SEND_FUNC_ARG", ALL_USED },
+#else
 	/*  185 */	{ "UNKNOWN [185]", ALL_USED },
+#endif
 	/*  186 */	{ "ISSET_ISEMPTY_THIS", ALL_USED },
 #endif
 #if PHP_VERSION_ID >= 70200
@@ -252,6 +255,23 @@ static const op_usage opcodes[] = {
 	/*  193 */	{ "GET_TYPE", ALL_USED },
 	/*  194 */	{ "FUNC_NUM_ARGS", ALL_USED },
 	/*  195 */	{ "FUNC_GET_ARGS", ALL_USED },
+	/*  196 */	{ "UNSET_CV", ALL_USED },
+	/*  197 */	{ "ISSET_ISEMPTY_CV", ALL_USED },
+#endif
+#if PHP_VERSION_ID >= 70300
+	/*  198 */	{ "FETCH_LIST_W", ALL_USED },
+#endif
+#if PHP_VERSION_ID >= 70400
+	/*  199 */  { "ARRAY_KEY_EXISTS", ALL_USED },
+	/*  200 */  { "ASSIGN_OBJ_REF", ALL_USED },
+	/*  201 */  { "ASSIGN_STATIC_PROP", ALL_USED },
+	/*  202 */  { "ASSIGN_STATIC_PROP_REF", ALL_USED },
+	/*  203 */  { "PRE_INC_STATIC_PROP", ALL_USED },
+	/*  204 */  { "PRE_DEC_STATIC_PROP", ALL_USED },
+	/*  205 */  { "POST_INC_STATIC_PROP", ALL_USED },
+	/*  206 */  { "POST_DEC_STATIC_PROP", ALL_USED },
+	/*  207 */  { "COPY_TMP", ALL_USED },
+	/*  208 */  { "ADD_ARRAY_UNPACK", ALL_USED },
 #endif
 };
 
@@ -390,7 +410,7 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsi
 			VLD_PRINT1(3, " IS_CONST (%d) ", VLD_ZNODE_ELEM(node, var) / sizeof(zval));
 #if PHP_VERSION_ID >= 70300
 			vld_dump_zval(*RT_CONSTANT((op_array->opcodes) + opline, node));
-#elif PHP_VERSION_ID >= 70000
+#else
 			vld_dump_zval(*RT_CONSTANT_EX(op_array->literals, node));
 #endif
 			break;
@@ -513,14 +533,23 @@ static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_a
 			}
 			break;
 		case ZEND_CATCH:
+#if PHP_VERSION_ID >= 70300
+			flags = OP1_USED;
+			if (op->extended_value & ZEND_LAST_CATCH) {
+				flags |= EXT_VAL;
+			} else {
+				flags |= OP2_USED|OP2_OPLINE;
+			}
+#else
 			flags = ALL_USED;
 			if (!op->result.num) {
-#if PHP_VERSION_ID >= 70100
+# if PHP_VERSION_ID >= 70100
 				flags |= EXT_VAL_JMP_REL;
-#else
+# else
 				flags |= EXT_VAL_JMP_ABS;
-#endif
+# endif
 			}
+#endif
 			break;
 	}
 	return flags;
@@ -532,7 +561,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 {
 	static uint last_lineno = (uint) -1;
 	int print_sep = 0, len;
-	char *fetch_type = "";
+	const char *fetch_type = "";
 	unsigned int flags, op1_type, op2_type, res_type;
 	const zend_op op = op_ptr[nr];
 
@@ -552,7 +581,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 
 	if (flags == SPECIAL) {
 		flags = vld_get_special_flags(&op, base_address);
-	} 
+	}
 	if (flags & OP1_OPLINE) {
 		op1_type = VLD_IS_OPLINE;
 	}
@@ -575,7 +604,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 		op2_type = VLD_IS_JMP_ARRAY;
 	}
 
-#if PHP_VERSION_ID >= 50600 && PHP_VERSION_ID < 70100
+#if PHP_VERSION_ID >= 70000 && PHP_VERSION_ID < 70100
 	switch (op.opcode) {
 		case ZEND_FAST_RET:
 			if (op.extended_value == ZEND_FAST_RET_TO_FINALLY) {
@@ -652,7 +681,15 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 	}
 
 	if (flags & EXT_VAL) {
+#if PHP_VERSION_ID >= 70300
+		if (op.opcode == ZEND_CATCH) {
+			vld_printf(stderr, "last ");
+		} else {
+			vld_printf(stderr, "%3d  ", op.extended_value);
+		}
+#else
 		vld_printf(stderr, "%3d  ", op.extended_value);
+#endif
 	} else {
 		vld_printf(stderr, "     ");
 	}
@@ -834,11 +871,16 @@ int vld_find_jumps(zend_op_array *opa, unsigned int position, size_t *jump_count
 	} else if (opcode.opcode == ZEND_CATCH) {
 		*jump_count = 2;
 		jumps[0] = position + 1;
-		if (!opcode.result.num) {
-#if PHP_VERSION_ID >= 70100
-			jumps[1] = position + (opcode.extended_value / sizeof(zend_op));
+#if PHP_VERSION_ID >= 70300
+		if (!(opcode.extended_value & ZEND_LAST_CATCH)) {
+			jumps[1] = VLD_ZNODE_JMP_LINE(opcode.op2, position, base_address);
 #else
+		if (!opcode.result.num) {
+# if PHP_VERSION_ID >= 70100
+			jumps[1] = position + (opcode.extended_value / sizeof(zend_op));
+# else
 			jumps[1] = opcode.extended_value;
+# endif
 #endif
 			if (jumps[1] == jumps[0]) {
 				jumps[1] = VLD_JMP_NOT_SET;
