@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 Derick Rethans                               |
+   | Copyright (c) 1997-2019 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -72,7 +72,7 @@
 #define VLD_IS_JMP_ARRAY  1<<26
 
 typedef struct _op_usage {
-	char *name;
+	const char  *name;
 	unsigned int flags;
 } op_usage;
 
